%% File: jordanholder.red
%% Date: April 10,1993
%% Status: 

%% This program computes a Jordan-Holder basis of a given nilpotent
%% Lie algebra given by its structure constants

%% Requirements:
%% workdimensiongn - the dimension
%% c(.,.,.) - array with the structure constants of the given
%% nilpotent matirix

%% Results:
%% newbasisgm(workdimension,workdimension) - the matrix giving the new
%%  basis in terms of the old one

%% The game is to succesively find that commutator [g,m], where g is
%% the Lie algebra, and m is a linear subspace. Generators for the linear
%% subspace m is stored in the matrix 
%% oldvectormatrix(workdimension,oldvectornolv), where oldvectornolv is 
%% the number of elements in a set of generators for m. Columns are the
%% coordinates with respect to the given basis in g. The resulting
%% generator of [g,m] is stored in
%% newvectormatrixlm(workdimension,newvectornolv).
%% We begin with the vectors X(m),...,X(1):
oldvectornolv:=workdimensiongv;
matrix oldvectormatrixlm(workdimensiongv,oldvectornolv);
for ilv:=1:workdimensiongv do 
 oldvectormatrixlm(workdimensiongv+1-ilv,ilv):=1;

collectnolv:=0;

While oldvectornolv neq 0 do
Begin

%% The bases for the various [g,m] are put in a row
   For ilv:=1:workdimensiongv do for ulv:=1:oldvectornolv do
   collectlo(ilv,collectnolv+ulv):=oldvectormatrixlm(ilv,ulv);
   collectnolv:=collectnolv+oldvectornolv;



%% The operator describing [g,m] is computed:
   testlv:=1;
   countlv:=0;
   For ilv:=1:workdimensiongv do for llv:=1:oldvectornolv do
   Begin
      If testlv neq 0 then countlv:=countlv+1;
      testlv:=0;
      For klv:=1:workdimensiongv do
      begin 
         clv:=for jlv:=1:workdimensiongv sum
          c(ilv,jlv,klv)*oldvectormatrixlm(jlv,llv);
         testlv:=testlv+clv**2;
         newvectorlo(klv,countlv):=clv;
      end;
   End;

%% So many new non-zero vectors are generated in [g,m]:
   newvectornolv:=countlv;
   write "Antal i ny vektor=",newvectornolv;
   matrix newvectormatrixlm(workdimensiongv,newvectornolv);
   for ilv:=1:workdimensiongv do for jlv:=1:newvectornolv do
    newvectormatrixlm(ilv,jlv):=newvectorlo(ilv,jlv);
   
%% We next proceed to compute a basis for [g,m]:
   semigramdimgv:=newvectornolv;
%% The matrix of inner products of vectors from [g,m] is computed:
   matrix semigramgm(semigramdimgv,semigramdimgv);
   semigramgm:=tp(newvectormatrixlm)*newvectormatrixlm;
%%   in "$HOME/REDUCE/LIE/CODE/GENERAL/udskift.red";
%% oldvectornolv:=gramdimgv;
%% Redefinition of oldvectornolv:
   oldvectornolv:=Udskift();
   write "Antal tilbage i ny vektor efter udskiftning=",oldvectornolv;
   showtime;

%% We have now found a basis for [g,m], namely a subset of the
%% generators found before and described by
%% jindgo(1),...,jindgl(oldvectornolv)     

%% We prepare for the next round:
   if oldvectornolv neq 0 then
   << matrix oldvectormatrixlm(workdimensiongv,oldvectornolv);
   for ilv:=1:workdimensiongv do for ulv:=1:oldvectornolv do
    oldvectormatrixlm(ilv,ulv):=newvectormatrixlm(ilv,jindgo(ulv));
   >>;

End;

   matrix collectlm(workdimensiongv,collectnolv);

%% We now put the whole thing together in reverse order:
   for ilv:=1:workdimensiongv do for jlv:=1:collectnolv do
   collectlm(ilv,jlv):=collectlo(ilv,collectnolv+1-jlv);

   semigramdimgv:=collectnolv;
   matrix semigramgm(semigramdimgv,semigramdimgv);

   semigramgm:=tp(collectlm)*collectlm;
%%   in "$HOME/REDUCE/LIE/CODE/GENERAL/udskift.red";
   Udskift(); 
  
   matrix newbasisgm(workdimensiongv,workdimensiongv);

   if gramdimgv neq workdimensiongv then write "Fejl" else
   for ilv:=1:workdimensiongv do for ulv:=1:workdimensiongv do
    newbasisgm(ilv,ulv):=collectlm(ilv,jindgo(ulv));

;end;










